#include	<windows.h>
#include	"resource.h"

#pragma comment(lib, "winmm.lib")

const unsigned notify_id = 0x100;

NOTIFYICONDATA nid =
{
	sizeof(NOTIFYICONDATA),            // cbSize
	0,                                 // hWnd
	notify_id,                         // uID
	NIF_ICON | NIF_MESSAGE | NIF_TIP,  // uFlags
	WM_USER,                           // uCallbackMessage
	0,                                 // hIcon,
	TEXT("MIDI to MOUSE"),             // szTip
};

NOTIFYICONDATA noteOn =
{
	sizeof(NOTIFYICONDATA), // cbSize
	0,                      // hWnd
	notify_id,              // uID
	NIF_ICON,               // uFlags
	0,                      // uCallbackMessage
	0,                      // hIcon,
	TEXT(""),               // szTip
};

NOTIFYICONDATA noteOff =
{
	sizeof(NOTIFYICONDATA), // cbSize
	0,                      // hWnd
	notify_id,              // uID
	NIF_ICON,               // uFlags
	0,                      // uCallbackMessage
	0,                      // hIcon,
	TEXT(""),               // szTip
};

HWND hWnd;
HINSTANCE hInstance;
HMIDIIN hMidiIn;

int lastNote = -1;

LRESULT CALLBACK WndProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_CREATE:
		noteOn.hWnd = noteOff.hWnd = nid.hWnd = hWnd;
		noteOff.hIcon = nid.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_NOTEOFF));
		noteOn.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_NOTEON));

		if (!Shell_NotifyIcon(NIM_ADD, &nid)){
			PostQuitMessage(GetLastError());
		}

		return 0;
	case WM_USER:
		if (hMidiIn != 0 && wParam == notify_id && lParam == WM_LBUTTONUP) {
			DestroyWindow(hWnd);
		}
		break;
	case MM_MIM_DATA:
		{
			BYTE midiStatus = LOBYTE(LOWORD(lParam));
			if (midiStatus != 0x80 && midiStatus != 0x90) {
				break;
			}

			BYTE midiNote = HIBYTE(LOWORD(lParam));
			BYTE midiVelocity = LOBYTE(HIWORD(lParam));

			if (midiStatus == 0x80 || midiVelocity == 0){
				if (lastNote == midiNote){
					lastNote = -1;
					mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, 0);
					Shell_NotifyIcon(NIM_MODIFY, &noteOff);
				}
			} else {
				if (lastNote != -1){
					mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, 0);
				}
				lastNote = midiNote;
				mouse_event(MOUSEEVENTF_LEFTDOWN, 0, 0, 0, 0);
				Shell_NotifyIcon(NIM_MODIFY, &noteOn);
			}
			break;
		}
	case WM_DESTROY:
		Shell_NotifyIcon(NIM_DELETE, &nid);

		if (hMidiIn != 0){
			midiInStop(hMidiIn);
			midiInReset(hMidiIn);
			midiInClose(hMidiIn);
		}
		PostQuitMessage(0);
		return 0;
	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

bool setMidiIn ()
{
	unsigned int maxMidiIn = midiInGetNumDevs();
	if (maxMidiIn == 0) {
		return false;
	}

	for (unsigned int i = 0; i < maxMidiIn; ++i){
		MIDIINCAPS caps;
		MMRESULT result = midiInGetDevCaps(i, &caps, sizeof(caps));
		if (result == MMSYSERR_NOERROR){
			switch (MessageBox(hWnd, caps.szPname, "Selection", MB_YESNOCANCEL))
			{
			case IDYES:
				break;
			case IDNO:
				continue;
			case IDCANCEL:
				return false;
			}
			result = midiInOpen(&hMidiIn, i, (DWORD_PTR)hWnd, 0, CALLBACK_WINDOW);
			if (result != MMSYSERR_NOERROR){
				TCHAR buf[512];
				midiInGetErrorText(result, buf, 512);
				MessageBox(hWnd, buf, "midiInGetDevCaps Error", MB_OK);
				return false;
			}
			result = midiInStart(hMidiIn);
			if (result != MMSYSERR_NOERROR){
				TCHAR buf[512];
				midiInGetErrorText(result, buf, 512);
				MessageBox(hWnd, buf, "midiInGetDevCaps Error", MB_OK);
				return false;
			}
			return true;
		} else {
			TCHAR buf[512];
			midiInGetErrorText(result, buf, 512);
			MessageBox(hWnd, buf, "midiInGetDevCaps Error", MB_OK);
		}
	}
	return false;
}

int WINAPI WinMain (HINSTANCE hInst, HINSTANCE, LPSTR, int)
{
	hInstance = hInst;

	WNDCLASS wc = { 0 };
	wc.lpfnWndProc = WndProc;
	wc.hInstance = hInst;
	wc.lpszClassName = "MIDItoMOUSE_WND";
	if (!RegisterClass(&wc)){
		return GetLastError();
	}

	hWnd = CreateWindow(wc.lpszClassName, TEXT(""), WS_OVERLAPPED, 0, 0, 0, 0, 0, 0, hInst, 0);

	if (!setMidiIn()){
		PostQuitMessage(-1);
	}

	MSG msg;
	int ret;
	while ((ret = GetMessage(&msg, 0, 0, 0)) != 0 && ret != -1)
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	if (ret == -1){
		return GetLastError();
	}

	return msg.wParam;
}

